@@ -19,6 +19,7 @@ module XMonad.Layout.ShowWName
     , showWName'
     , defaultSWNConfig
     , SWNConfig(..)
+    , ShowWName
     ) where
 
 import XMonad
@@ -96,7 +97,7 @@ flashName c (Rectangle _ _ wh ht) wrs = do
       x     = (fi wh - width + 2) `div` 2
   w <- createNewWindow (Rectangle (fi x) (fi y) (fi width) (fi hight)) Nothing "" True
   showWindow w
-  paintAndWrite w f (fi width) (fi hight) 0 "" "" (swn_color c) (swn_bgcolor c) AlignCenter n
+  paintAndWrite w f (fi width) (fi hight) 0 "" "" (swn_color c) (swn_bgcolor c) [AlignCenter] [n]
   releaseXMF f
   io $ sync d False
   i <- startTimer (swn_fade c)
