@@ -29,7 +29,8 @@ module XMonad.Prompt
     , defaultXPKeymap
     , quit
     , killBefore, killAfter, startOfLine, endOfLine
-    , pasteString, copyString, moveCursor
+    , pasteString, moveCursor
+    , setInput, getInput
     , moveWord, killWord, deleteString
     , moveHistory, setSuccess, setDone
     , Direction1D(..)
@@ -51,39 +52,43 @@ module XMonad.Prompt
     , splitInSubListsAt
     , breakAtSpace
     , uniqSort
-    , decodeInput
-    , encodeOutput
     , historyCompletion
     , historyCompletionP
     -- * History filters
     , deleteAllDuplicates
     , deleteConsecutive
+    , HistoryMatches
+    , initMatches
+    , historyUpMatching
+    , historyDownMatching
+    -- * Types
+    , XPState
     ) where
 
 import Prelude hiding (catch)
 
-import XMonad  hiding (config, io, numlockMask, cleanMask)
-import qualified XMonad as X (numlockMask,config)
+import XMonad  hiding (config, cleanMask)
+import qualified XMonad as X (numberlockMask)
 import qualified XMonad.StackSet as W
 import XMonad.Util.Font
 import XMonad.Util.Types
-import XMonad.Util.XSelection (getSelection, putSelection)
+import XMonad.Util.XSelection (getSelection)
 
+import Codec.Binary.UTF8.String (decodeString)
+import Control.Applicative ((<$>))
 import Control.Arrow ((&&&),first)
 import Control.Concurrent (threadDelay)
-import Control.Monad.Reader
+import Control.Exception.Extensible hiding (handle)
 import Control.Monad.State
-import Control.Applicative ((<$>))
-import Data.Char
 import Data.Bits
-import Data.Maybe
+import Data.Char (isSpace)
+import Data.IORef
 import Data.List
+import Data.Maybe (fromMaybe)
 import Data.Set (fromList, toList)
-import System.Directory
+import System.Directory (getAppUserDataDirectory)
 import System.IO
 import System.Posix.Files
-import Control.Exception.Extensible hiding (handle)
-
 import qualified Data.Map as M
 
 -- $usage
@@ -136,7 +141,10 @@ data XPConfig =
         , defaultText       :: String     -- ^ The text by default in the prompt line
         , autoComplete      :: Maybe Int  -- ^ Just x: if only one completion remains, auto-select it,
         , showCompletionOnTab :: Bool     -- ^ Only show list of completions when Tab was pressed
-                                         --   and delay by x microseconds
+                                          --   and delay by x microseconds
+        , searchPredicate   :: String -> String -> Bool
+                                          -- ^ Given the typed string and a possible
+                                          --   completion, is the completion valid?
         }
 
 data XPType = forall p . XPrompt p => XPT p
@@ -210,15 +218,17 @@ defaultXPConfig =
         , historyFilter     = id
         , defaultText       = []
         , autoComplete      = Nothing
-        , showCompletionOnTab = False }
-greenXPConfig = defaultXPConfig { fgColor = "green", bgColor = "black" }
+        , showCompletionOnTab = False
+        , searchPredicate   = isPrefixOf
+        }
+greenXPConfig = defaultXPConfig { fgColor = "green", bgColor = "black", promptBorderWidth = 0 }
 amberXPConfig = defaultXPConfig { fgColor = "#ca8f2d", bgColor = "black", fgHLight = "#eaaf4c" }
 
 type ComplFunction = String -> IO [String]
 
 initState :: XPrompt p => Display -> Window -> Window -> Rectangle -> ComplFunction
-          -> GC -> XMonadFont -> p -> [String] -> XPConfig -> XPState
-initState d rw w s compl gc fonts pt h c =
+          -> GC -> XMonadFont -> p -> [String] -> XPConfig -> KeyMask -> XPState
+initState d rw w s compl gc fonts pt h c nm =
     XPS { dpy                = d
         , rootw              = rw
         , win                = w
@@ -237,7 +247,7 @@ initState d rw w s compl gc fonts pt h c =
         , config             = c
         , successful         = False
         , done               = False
-        , numlockMask        = X.numlockMask defaultConfig
+        , numlockMask        = nm
         }
 
 -- this would be much easier with functional references
@@ -247,6 +257,15 @@ command = W.focus . commandHistory
 setCommand :: String -> XPState -> XPState
 setCommand xs s = s { commandHistory = (commandHistory s) { W.focus = xs }}
 
+-- | Sets the input string to the given value.
+setInput :: String -> XP ()
+setInput = modify . setCommand
+
+-- | Returns the current input string. Intented for use in custom keymaps
+-- where the 'get' or similar can't be used to retrieve it.
+getInput :: XP String
+getInput = gets command
+
 -- | Same as 'mkXPrompt', except that the action function can have
 --   type @String -> X a@, for any @a@, and the final action returned
 --   by 'mkXPromptWithReturn' will have type @X (Maybe a)@.  @Nothing@
@@ -255,30 +274,28 @@ setCommand xs s = s { commandHistory = (commandHistory s) { W.focus = xs }}
 --   module.
 mkXPromptWithReturn :: XPrompt p => p -> XPConfig -> ComplFunction -> (String -> X a)  -> X (Maybe a)
 mkXPromptWithReturn t conf compl action = do
-  c <- ask
-  let d = display c
-      rw = theRoot c
-  s <- gets $ screenRect . W.screenDetail . W.current . windowset
-  hist <- liftIO readHistory
-  w <- liftIO $ createWin d rw conf s
-  liftIO $ selectInput d w $ exposureMask .|. keyPressMask
-  gc <- liftIO $ createGC d w
-  liftIO $ setGraphicsExposures d gc False
+  XConf { display = d, theRoot = rw } <- ask
+  s    <- gets $ screenRect . W.screenDetail . W.current . windowset
+  hist <- io readHistory
+  w    <- io $ createWin d rw conf s
+  io $ selectInput d w $ exposureMask .|. keyPressMask
+  gc <- io $ createGC d w
+  io $ setGraphicsExposures d gc False
   fs <- initXMF (font conf)
-  numlock <- asks $ X.numlockMask . X.config
+  numlock <- gets $ X.numberlockMask
   let hs = fromMaybe [] $ M.lookup (showXPrompt t) hist
-      st = (initState d rw w s compl gc fs (XPT t) hs conf)
-           { numlockMask = numlock }
-  st' <- liftIO $ execStateT runXP st
+      st = initState d rw w s compl gc fs (XPT t) hs conf numlock
+  st' <- io $ execStateT runXP st
 
   releaseXMF fs
-  liftIO $ freeGC d gc
+  io $ freeGC d gc
   if successful st'
     then do
-      liftIO $ writeHistory $ M.insertWith
-                                (\xs ys -> take (historySize conf)
-                                        . historyFilter conf $ xs ++ ys)
-                                (showXPrompt t) (historyFilter conf [command st'])
+      let prune = take (historySize conf)
+      io $ writeHistory $ M.insertWith
+                                (\xs ys -> prune . historyFilter conf $ xs ++ ys)
+                                (showXPrompt t)
+                                (prune $ historyFilter conf [command st'])
                                 hist
                                 -- we need to apply historyFilter before as well, since
                                 -- otherwise the filter would not be applied if
@@ -302,8 +319,7 @@ mkXPrompt t conf compl action = mkXPromptWithReturn t conf compl action >> retur
 
 runXP :: XP ()
 runXP = do
-  st <- get
-  let (d,w) = (dpy &&& win) st
+  (d,w) <- gets (dpy &&& win)
   status <- io $ grabKeyboard d w True grabModeAsync grabModeAsync currentTime
   when (status == grabSuccess) $ do
           updateWindows
@@ -403,13 +419,13 @@ defaultXPKeymap = M.fromList $
   , (xK_a, startOfLine)
   , (xK_e, endOfLine)
   , (xK_y, pasteString)
-  , (xK_c, copyString)
   , (xK_Right, moveWord Next)
   , (xK_Left, moveWord Prev)
   , (xK_Delete, killWord Next)
   , (xK_BackSpace, killWord Prev)
   , (xK_w, killWord Prev)
-  , (xK_q, quit)
+  , (xK_g, quit)
+  , (xK_bracketleft, quit)
   ] ++
   map (first $ (,) 0)
   [ (xK_Return, setSuccess True >> setDone True)
@@ -428,13 +444,13 @@ defaultXPKeymap = M.fromList $
 keyPressHandle :: KeyMask -> KeyStroke -> XP ()
 keyPressHandle m (ks,str) = do
   km <- gets (promptKeymap . config)
-  mask <- cleanMask m
-  case M.lookup (mask,ks) km of
+  kmask <- cleanMask m -- mask is defined in ghc7
+  case M.lookup (kmask,ks) km of
     Just action -> action >> updateWindows
     Nothing -> case str of
                  "" -> eventLoop handle
-                 _ -> when (mask .&. controlMask == 0) $ do
-                                 insertString (decodeInput str)
+                 _ -> when (kmask .&. controlMask == 0) $ do
+                                 insertString (decodeString str)
                                  updateWindows
                                  completed <- tryAutoComplete
                                  when completed $ setSuccess True >> setDone True
@@ -505,10 +521,6 @@ insertString str =
 pasteString :: XP ()
 pasteString = join $ io $ liftM insertString getSelection
 
--- | Copy the currently entered string into the X selection.
-copyString :: XP ()
-copyString = gets command >>= io . putSelection
-
 -- | Remove a character at the cursor position
 deleteString :: Direction1D -> XP ()
 deleteString d =
@@ -724,28 +736,15 @@ redrawComplWin compl = do
 
 printComplList :: Display -> Drawable -> GC -> String -> String
                -> [Position] -> [Position] -> [[String]] -> XP ()
-printComplList _ _ _ _ _ _ _ [] = return ()
-printComplList _ _ _ _ _ [] _ _ = return ()
-printComplList d drw gc fc bc (x:xs) y (s:ss) = do
-  printComplColumn d drw gc fc bc x y s
-  printComplList d drw gc fc bc xs y ss
-
-printComplColumn :: Display -> Drawable -> GC -> String -> String
-                 -> Position -> [Position] -> [String] -> XP ()
-printComplColumn _ _ _ _ _ _ _ [] = return ()
-printComplColumn _ _ _ _ _ _ [] _ = return ()
-printComplColumn d drw gc fc bc x (y:yy) (s:ss) = do
-  printComplString d drw gc fc bc x y s
-  printComplColumn d drw gc fc bc x yy ss
-
-printComplString :: Display -> Drawable -> GC -> String -> String
-                 -> Position -> Position -> String  -> XP ()
-printComplString d drw gc fc bc x y s = do
-  st <- get
-  if completionToCommand (xptype st) s == commandToComplete (xptype st) (command st)
-     then printStringXMF d drw (fontS st) gc
-                            (fgHLight $ config st) (bgHLight $ config st) x y s
-     else printStringXMF d drw (fontS st) gc fc bc x y s
+printComplList d drw gc fc bc xs ys sss =
+    zipWithM_ (\x ss ->
+        zipWithM_ (\y s -> do
+            st <- get
+            let (f,b) = if completionToCommand (xptype st) s == commandToComplete (xptype st) (command st)
+                            then (fgHLight $ config st,bgHLight $ config st)
+                            else (fc,bc)
+            printStringXMF d drw (fontS st) gc f b x y s)
+        ys ss) xs sss
 
 -- History
 
@@ -768,7 +767,9 @@ readHistory = readHist `catch` \(SomeException _) -> return emptyHistory
 writeHistory :: History -> IO ()
 writeHistory hist = do
   path <- getHistoryFile
-  writeFile path (show hist) `catch` \(SomeException _) -> hPutStrLn stderr "error in writing"
+  let filtered = M.filter (not . null) hist
+  writeFile path (show filtered) `catch` \(SomeException e) ->
+                          hPutStrLn stderr ("error writing history: "++show e)
   setFileMode path mode
     where mode = ownerReadMode .|. ownerWriteMode
 
@@ -834,14 +835,6 @@ getNextCompletion c l = l !! idx
                   Just i  -> if i >= length l - 1 then 0 else i + 1
                   Nothing -> 0
 
--- Lift an IO action into the XP
-io :: IO a -> XP a
-io = liftIO
-
--- Shorthand for fromIntegral
-fi :: (Num b, Integral a) => a -> b
-fi = fromIntegral
-
 -- | Given a maximum length, splits a list into sublists
 splitInSubListsAt :: Int -> [a] -> [[a]]
 splitInSubListsAt _ [] = []
@@ -889,3 +882,45 @@ uniqSort = toList . fromList
 deleteAllDuplicates, deleteConsecutive :: [String] -> [String]
 deleteAllDuplicates = nub
 deleteConsecutive = map head . group
+
+newtype HistoryMatches = HistoryMatches (IORef ([String],Maybe (W.Stack String)))
+
+-- | Initializes a new HistoryMatches structure to be passed
+-- to historyUpMatching
+initMatches :: (Functor m, MonadIO m) => m HistoryMatches
+initMatches = HistoryMatches <$> liftIO (newIORef ([],Nothing))
+
+historyNextMatching :: HistoryMatches -> (W.Stack String -> W.Stack String) -> XP ()
+historyNextMatching hm@(HistoryMatches ref) next = do
+  (completed,completions) <- io $ readIORef ref
+  input <- getInput
+  if input `elem` completed
+     then case completions of
+            Just cs -> do
+                let cmd = W.focus cs
+                modify $ setCommand cmd
+                modify $ \s -> s { offset = length cmd }
+                io $ writeIORef ref (cmd:completed,Just $ next cs)
+            Nothing -> return ()
+     else do -- the user typed something new, recompute completions
+       io . writeIORef ref . ((,) [input]) . filterMatching input =<< gets commandHistory
+       historyNextMatching hm next
+    where filterMatching :: String -> W.Stack String -> Maybe (W.Stack String)
+          filterMatching prefix = W.filter (prefix `isPrefixOf`) . next
+
+-- | Retrieve the next history element that starts with
+-- the current input. Pass it the result of initMatches
+-- when creating the prompt. Example:
+--
+-- > ..
+-- > ((modMask,xK_p), shellPrompt . myPrompt =<< initMatches)
+-- > ..
+-- > myPrompt ref = defaultPrompt
+-- >   { promptKeymap = M.union [((0,xK_Up), historyUpMatching ref)
+-- >                            ,((0,xK_Down), historyDownMatching ref)]
+-- >                            (promptKeymap defaultPrompt)
+-- >   , .. }
+--
+historyUpMatching, historyDownMatching :: HistoryMatches -> XP ()
+historyUpMatching hm = historyNextMatching hm W.focusDown'
+historyDownMatching hm = historyNextMatching hm W.focusUp'
