@@ -4,7 +4,7 @@
 -- Copyright   :  (C) 2007 Andrea Rossato, David Roundy
 -- License     :  BSD3
 --
--- Maintainer  :  
+-- Maintainer  :
 -- Stability   :  unstable
 -- Portability :  unportable
 --
@@ -15,7 +15,10 @@
 module XMonad.Prompt.Workspace (
                              -- * Usage
                              -- $usage
-                             workspacePrompt
+                             workspacePrompt,
+
+                             -- * For developers
+                             Wor(Wor),
                               ) where
 
 import XMonad hiding ( workspaces )
